from openfisca_core.reforms import Reform
from openfisca_core.parameters import ParameterNode

from ..contrefactuel_plf import modify_parameters as modify_parameters_contrefacturel
from ..inflaters import inflate_parameters

from openfisca_core.model_api import *
from openfisca_france.model.base import *

import os

reform_date = "2026-01-01"


def modify_parameters(parameters):
    parameters = modify_parameters_contrefacturel(parameters)

    inflateurs = ParameterNode(
        "inflateurs",
        directory_path=os.path.join(os.path.dirname(__file__), "parameters/inflateurs/"),
    )

    parameters.children['inflateurs'].children['inflateur_impot'] = inflateurs.inflateur_impot
    parameters.children['inflateurs'].children['inflateur_bmaf'] = inflateurs.inflateur_bmaf

    param_ir = parameters.impot_revenu
    liste_param_impot = [
        ## revalo sans texte de loi mais usuelle :
        param_ir.bareme_ir_depuis_1945.bareme,
        param_ir.calcul_impot_revenu.plaf_qf.plafond_avantages_procures_par_demi_part,
        param_ir.calcul_impot_revenu.plaf_qf.decote.seuil_couple,
        param_ir.calcul_impot_revenu.plaf_qf.decote.seuil_celib,
        param_ir.calcul_revenus_imposables.abat_rni.enfant_marie,
        param_ir.calcul_revenus_imposables.charges_deductibles.pensions_alimentaires.plafond,
        ## revalo selon 1e tranche de l'IR
        param_ir.calcul_revenus_imposables.deductions.abatpen,
        param_ir.calcul_revenus_imposables.deductions.abatpro,
        param_ir.calcul_revenus_imposables.charges_deductibles.accueil_personne_agee.plafond,
        param_ir.calcul_revenus_imposables.abat_rni.contribuable_age_invalide,
    ]
    for noeud in liste_param_impot:
        inflate_parameters(
            noeud,
            inflator=parameters(reform_date).inflateurs.inflateur_impot,
            base_year=2024,
            last_year=2025,
            start_instant="2025-01-01",
            round_ndigits=0,
        )

    inflate_parameters(
        parameters.prestations_sociales.prestations_familiales.bmaf.bmaf,
        inflator=parameters(reform_date).inflateurs.inflateur_bmaf,
        base_year=2025,
        last_year=2026,
        start_instant="2026-04-01",
        round_ndigits=2,
    )

class PlfPlfss2026(Reform):
    name = "PLF & PLFSS 2026"
    tax_benefit_system_name = "openfisca_france_with_indirect_taxation"
    
    def apply(self):
        self.modify_parameters(modifier_function=modify_parameters)
